function [ features timemarks events ] = asbsfeatures( breathfile,...
                                            feature, marked )
%%
% ASBSFEATURES(BREATHFILE,WS,AWO,TH,FEATURE,MARKS)
%
% Purpose:  returns chosen features, time marks and event of each
%           obtained frame. Time marks are available if related
%           file exists.
%
% Inputs:   breathfile  - file with breathing 
%           feature     - feature(s) to extract
%           marked      - true if exist timemark file
%           
%
% Output:   features    - matrix (totalwindows, ASBS_TOTAL_FEATURES)
%                         with selectec features
%           timemarks   - matrix (totalwindows, 2) with start end time
%                         of each windowframe
%           events      - vector (totalwindows, 1) with associated
%                         event  according to marks
%
% Author:   juan a. Castro
% Version:  1.0 jan/2015
%%

global ASBS_SAMPLING;
global ASBS_OVERLAPPED;
global ASBS_THRESHOLD;
global ASBS_WINDOW_SAMPLES;
global ASBS_TOTAL_FEATURES;

% append extension if 
[path name ext] = fileparts (breathfile);
if isempty( ext )
    breathfile = [breathfile '.wav'];
end

% Read breath file
[x, fs, nbits] = wavread(breathfile); 

if fs ~= ASBS_SAMPLING
    disp('Sampling does not match global setting');
    return
end

if marked
    % Read marks file
    str = strfind(breathfile,'_');
    marks = asbsimportxls([breathfile(1:str(1)-1) '.xlsx']);
end

if ASBS_OVERLAPPED, wpf = 2; else wpf = 1; end

% number of points in window
step = floor (ASBS_WINDOW_SAMPLES / wpf);

% loop x framed 
totalwindows = floor(length(x) / step);

% set output dimensions
features        = zeros (totalwindows, ASBS_TOTAL_FEATURES);
timemarks       = zeros (totalwindows, 2);
events          = zeros (totalwindows, 1);

imark = 1;
j = 1;
for i = 0 : totalwindows - wpf

    a = i * step + 1;
    b = i * step + ASBS_WINDOW_SAMPLES;

    % Calculates frame energy
    e = asbssignalrms( x(a:b) );

    % chech if we need to process the frame
    if (e >= ASBS_THRESHOLD)

        if marked
            %% ASSIGN TIME MARKS
            timea = a / fs;
            timeb = b / fs;
            timemarks (j, 1) = timea;
            timemarks (j, 2) = timeb;
        
            %% ASSIGN EVENT
            while marks (imark, 1) < timeb
                imark = imark + 1;
            end
            if marks(imark, 1) == timeb
                events(j) = 0;
            elseif timea < marks(imark - 1, 1)
                events (j) = 0;
            else
                events (j) = marks(imark, 2);
            end
        end
        features (j, :) = asbsframefeatures(x(a:b), feature);

        j = j + 1;

    end % if energy > threshold

end % window loop

features (j:end, :) = [];
timemarks(j:end, :) = [];
events   (j:end, :) = [];

%% EOF
