function asbslabconsole( model, features, varargin )
%%
% ASBSLABCONSOLE(MODEL,FEATURES)
%
% Purpose:  Identifies breathing events from default Audio Input Device
%           filename (included in varargin
%
% Inputs:   model       - repository version (specific fs and ws) 
%           features    - features taken into account 
%           varargin    - the wav file in case identification will use
%                         the file instead of microphone
%           
% Author:   juan a. Castro
% Version:  1.0 jan/2015
%%

global ASBS_SAMPLING;
global ASBS_WINDOW_SAMPLES;
global ASBS_OVERLAPPED;
global ASBS_THRESHOLD;
global ASBS_ZMODEL;
global ASBS_I_EVENT;
global ASBS_MU;
global ASBS_DELTA;
global ASBS_KNNOBJ;
global ASBS_ELLIP_ZEROS;
global ASBS_ELLIP_POLES;

asbsloadmodel( model );

if nargin == 2  % Uses mcrophone
    AR = dsp.AudioRecorder;
    AR.SampleRate = ASBS_SAMPLING;
    AR.NumChannels = 1;
elseif nargin == 3 % Read a file
        % append extension if 
    [path name ext] = fileparts (varargin{1});
    if isempty( ext )
        filename = [varargin{1} '.wav'];
    end
    if exist(filename, 'file') == false
        disp ('wav file does not exist');
        return;
    end
    AR = dsp.AudioFileReader(filename);
    if AR.SampleRate ~= ASBS_SAMPLING
        disp('Sample rate does nt match with model');
        return;
    end
    AR.OutputDataType = 'double';
else
    disp ('Invalid number or input arguments');
    return;
end

SPF = AR.SamplesPerFrame;

if ASBS_OVERLAPPED, wpf = 2; else wpf = 1; end

% number of points in window
SS = ASBS_WINDOW_SAMPLES / wpf;

asbscreateknnobj(features);
IFeatures = asbsfeatureindex (features);

asbsstminit();

buffer = zeros(4096,1);

if nargin == 3
%% IDENTIFYING FROM A FILE
disp('Monitoring now...');
    BI = 1;
    tic;
    while ~isDone(AR)
        while BI <= ASBS_WINDOW_SAMPLES
            buffer(BI:BI+SPF-1) = step(AR);
            BI = BI + SPF;
        end
        x = filter (ASBS_ELLIP_ZEROS, ASBS_ELLIP_POLES,...
                    buffer(1:ASBS_WINDOW_SAMPLES));    
        % Calculates frame energy
        e = asbssignalrms( x(1:ASBS_WINDOW_SAMPLES) );

        % chech if we need to process the frame
        if (e >= ASBS_THRESHOLD)
            % Get features

            zquery = asbsframefeatures(x(1:ASBS_WINDOW_SAMPLES),...
                                        features);
            zquery = bsxfun(@rdivide, ...
                        bsxfun(@minus,zquery, ASBS_MU),ASBS_DELTA);
            IDX = knnsearch(ASBS_KNNOBJ, zquery(:,IFeatures));
            value = ASBS_ZMODEL(IDX,ASBS_I_EVENT);
        else
            value = 0;
            tic;
        end % if energy > threshold
        [ev p ] = asbsstmactions(value, e, toc);
        if toc > 4
            tic;
            fprintf ('\nToo much noise to detect breathing');
            ev = 0;
            asbsstminit();
        end
        if ev > 0
            switch ev
                case 1
                    str = 'mouth inspiration';
                case 2
                    str = 'mouth espiration';
                case 3
                    str = 'nasal inspiration';
                case 4
                    str = 'nasal espiration';
            end
            fprintf('\n%s with %.2f %% of reliability', str, p*100);
        end
    
        buffer(1:BI-SS) = buffer(SS+1:BI);
        BI = BI - SS;
    
    end 
    fprintf('\n');
    release (AR);
    
    return;
end

%% IDENTIFYING FROM MICROPHONE
disp('Monitoring now... Press CTRL+C to stop.');
BI = 1;
tic;
while true
    while BI <= ASBS_WINDOW_SAMPLES
        buffer(BI:BI+SPF-1) = step(AR);
        BI = BI + SPF;
    end
    
    x = filter (ASBS_ELLIP_ZEROS, ASBS_ELLIP_POLES, ...
                    buffer(1:ASBS_WINDOW_SAMPLES));    
    % Calculates frame energy
    e = asbssignalrms( x(1:ASBS_WINDOW_SAMPLES) );

    % chech if we need to process the frame
    if (e >= ASBS_THRESHOLD)
        % Get features
        zquery = asbsframefeatures(x(1:ASBS_WINDOW_SAMPLES),...
                                features);
        zquery = bsxfun(@rdivide, ...
                        bsxfun(@minus,zquery, ASBS_MU),ASBS_DELTA);
        IDX = knnsearch(ASBS_KNNOBJ, zquery(:,IFeatures));
        value = ASBS_ZMODEL(IDX,ASBS_I_EVENT);
    else
        value = 0;
        tic;
    end % if energy > threshold
    [ev p ] = asbsstmactions(value, e, toc);
    if toc > 4
        tic;
        fprintf ('\nToo much noise to detect breathing');
        ev = 0;
    end

    if ev > 0
        switch ev
            case 1
                str = 'mouth inspiration';
            case 2
                str = 'mouth espiration';
            case 3
                str = 'nasal inspiration';
            case 4
                str = 'nasal espiration';
        end
        fprintf('\n%s with %.2f %% of reliability', str, p*100);
    end
    
    buffer(1:BI-SS) = buffer(SS+1:BI);
    BI = BI - SS;
    
end % window loop
fprintf('\n');

release(AR);

%% EOF
