function events = asbspredict( breathfile, features )
%%
% ASBSPREDICT(BREATHFILE,FEATURES)
%
% Purpose:  Performs breathing event prediction from breathfile
%           using features
%           from features. Call to ASBSLOADMODEL must be done first.
%
% Inputs:   breathfile  - file with breath sound 
%           feature     - feature(s) to extract
%           
%
% Output:   events      - column1: predicted event
%                       - column2: % of reliability
%
% Author:   juan a. Castro
% Version:  1.0 jan/2015
%%

global ASBS_SAMPLING;
global ASBS_WINDOW_SAMPLES;
global ASBS_OVERLAPPED;
global ASBS_THRESHOLD;
global ASBS_MODELCODE;
global ASBS_ZMODEL;
global ASBS_I_EVENT;
global ASBS_MU;
global ASBS_DELTA;
global ASBS_KNNOBJ;

% append extension if 
[path name ext] = fileparts (breathfile);
if isempty( ext )
    breathfile = [breathfile '.wav'];
end

% Read breath file
[x, fs, nbits] = wavread(breathfile); 

if fs ~= ASBS_SAMPLING
    disp('Sampling does not match global setting');
    return
end

if isempty(ASBS_MODELCODE)
    disp ('No model selected');
    return
end

if ASBS_OVERLAPPED, wpf = 2; else wpf = 1; end

% number of points in window
step = floor (ASBS_WINDOW_SAMPLES / wpf);

% loop x framed 
totalwindows = floor(length(x) / step);

asbscreateknnobj(features);
IFeatures = asbsfeatureindex (features);

asbsstminit();

j = 1;
for i = 0 : totalwindows - wpf

    a = i * step + 1;
    b = i * step + ASBS_WINDOW_SAMPLES;

    % Calculates frame energy
    e = asbssignalrms( x(a:b) );

    % chech if we need to process the frame
    if (e >= ASBS_THRESHOLD)
        % Get features
        zquery = asbsframefeatures(x(a:b), features);
        zquery = bsxfun(@rdivide, ...
                    bsxfun(@minus,zquery, ASBS_MU),ASBS_DELTA);
        IDX = knnsearch(ASBS_KNNOBJ, zquery(:,IFeatures));
        value = ASBS_ZMODEL(IDX,ASBS_I_EVENT);
    else
        value = 0;
    end % if energy > threshold
    [ev p ] = asbsstmactions(value, e, b / fs);
    if ev > 0
        events (j, 1) = ev;
        events (j, 2) = p;
        j = j + 1;
    end    
end % window loop

%% EOF
